@@ -5,7 +5,7 @@ from lihil.routing import EndpointProps
 async def test_route_merge_endpoint_plugin():
     called: list[str] = []
 
-    async def dummy_plugin(*args):
+    def dummy_plugin(*args):
         called.append("plugin called")
 
     route = Route(props=EndpointProps(plugins=[dummy_plugin]))
@@ -14,8 +14,6 @@ async def test_route_merge_endpoint_plugin():
 
     route.get(dummy_handler, plugins=[dummy_plugin])
 
-    await route.setup()
-
     ep = route.get_endpoint(dummy_handler)
 
     # merged plugin from route and endpoint so we have two plugins
